@@ -26,30 +26,52 @@
 # policies, either expressed or implied, of James Yates Farrimond.
 
 '''
-Cookbooks are sets of recipes to apply to a server to create something that is
-made up of many packages.
+Cookbooks are sets of recipes to apply to a server to create systems made up of
+subsystems.
 '''
 
 class Cookbook(object):
     '''
-    Base object for all cookbooks to be created from.  Subclass this for your
-    own cookbooks.  Generally you'll just define a list of recipes to run in
-    recipe_list.  If you over-ride any of the functions, be sure to call the
-    base class functions so that all the good stuff still happens.
+    The Cookbook class is the base class for all cookbooks to be created from.
+    If you over-ride any of the functions, be sure to call the base class
+    functions so that all the good stuff still happens.
 
-    In your overridden class you'll fill recipe_list with class objects for
-    recipes that you want run when applying the cookbook.  It's best to define
-    this at the class level.  The recipes will be applied in the order they're
-    defined in the list.
+    In your subclass you will fill the class-level variable recipe_list with
+    class objects for recipes that you want to run when applying the cookbook.
+    The recipes will be applied in the order they're defined in the list.
 
-    example::
+    Example::
 
-      recipe_list = [AwesomeRecipe, WayCoolRecipe]
+      from frycook import Cookbook
+      from recipes import AwesomeRecipe, WayCoolRecipe
+
+      class MyOwnCookbook(Cookbook):
+
+          recipe_list = [AwesomeRecipe,
+                         WayCoolRecipe]
+
+    All cookbook files should live in a single cookbooks package.  The
+    __init__.py file for this package should import all cookbooks and have a
+    list of them called cookbooks so they can easily be imported by the
+    frycooker program.
+
+    Example::
+
+      from base import CookbookBase
+      from web import CookbookWeb
+
+      cookbooks = {
+          'base': CookbookBase,
+          'web': CookbookWeb
+      }
     '''
     recipe_list = []
 
     def __init__(self, settings, environment):
         '''
+        Initialize the cookbook object with the settings and environment
+        dictionaries.
+
         @type settings: dict
 
         @param settings: settings dictionary
@@ -67,19 +89,37 @@ class Cookbook(object):
     #######################
 
     def handle_pre_apply_messages(self, computer):
+        '''
+        Run the pre_apply_message functions for all the recipes defined in
+        recipe_list. Override this if there are cookbook-level messages you
+        would like to add.  Be sure to call the base if you override this.
+
+        @type computer: string
+
+        @param computer: computer to apply recipe checks to
+        '''
         for recipe in self.recipes:
             recipe.handle_pre_apply_message(computer)
 
     def handle_post_apply_messages(self, computer):
+        '''
+        Run the post_apply_message functions for all the recipes defined in
+        recipe_list. Override this if there are cookbook-level messages you
+        would like to add.  Be sure to call the base if you override this.
+
+        @type computer: string
+
+        @param computer: computer to apply recipe checks to
+        '''
         for recipe in self.recipes:
             recipe.handle_post_apply_message(computer)
 
     def pre_apply_checks(self, computer):
         '''
-        Runs the pre_apply_checks functions for all the recipes defined in
+        Run the pre_apply_checks functions for all the recipes defined in
         recipe_list. Override this if there's something you need to check above
         and beyond the recipe-level data.  Be sure to call the base if you
-        subclass this.
+        override this.
 
         @type computer: string
 
@@ -90,9 +130,9 @@ class Cookbook(object):
 
     def apply(self, computer):
         '''
-        Runs the apply functions for all the recipes defined in recipe_list.
+        Run the apply functions for all the recipes defined in recipe_list.
         Override this if there's something you need to do besides just running
-        all the recipes.  Be sure to call the base class if you subclass this.
+        all the recipes.  Be sure to call the base class if you override this.
 
         @type computer: string
 
@@ -116,21 +156,15 @@ class Cookbook(object):
         self.handle_post_apply_messages(computer)
 
     def run_messages(self, computer):
-        self.handle_pre_apply_messages(computer)
-        self.handle_post_apply_messages(computer)
-
-    #########################
-    ######## CLEANUP ########
-    #########################
-
-    def run_cleanup(self, computer):
         '''
-        Run the cleanup process for the computer.  This is usually just called
-        from frycooker.
+        Run the handle_pre_apply_messages and handle_post_apply_messages
+        functions for all the recipes defined in recipe_list. This is useful to
+        display all the possible messages to the user without actually applying
+        the cookbook.  This is usually just called from frycooker.
 
         @type computer: string
 
-        @param computer: computer to apply recipe cleanup to
+        @param computer: computer to show messages for
         '''
-        for recipe in self.recipes:
-            recipe.run_cleanup(computer)
+        self.handle_pre_apply_messages(computer)
+        self.handle_post_apply_messages(computer)
