@@ -26,9 +26,9 @@
 # policies, either expressed or implied, of James Yates Farrimond.
 
 '''
-Recipes are the basic units of setup in frycook.  Generally a recipe
-corresponds to a package that needs to be installed or a subsystem that needs
-to be configured.
+Recipes define subsystems that are distinct parts of larger systems.  They are
+the basic units of setup in frycook.  Generally a recipe corresponds to a
+package that needs to be installed or configured.
 
 idempotence
 ===========
@@ -46,7 +46,6 @@ could cause undesired configuration changes or expensive operations that you
 don't want to happen every time.  These functions are a huge aid in writing
 idempotent recipes and cookbooks.
 
-
 apply
 =====
 
@@ -54,33 +53,25 @@ This is where you apply a recipe to a server.  There are two class methods that
 get called during the apply process, and possibly two messages that get
 displayed.  Generally you'll just override the apply method and sometimes add
 pre_apply or post_apply messages.  If you override pre_apply_checks, remember
-to call the base class method.  Here's the order that things happen:
+to call the base class method.  Here's the order that things happen in:
 
 pre_apply_message -> pre_apply_checks() -> apply() -> post_apply_message
 
-cleanup
-=======
-
-This is where you cleanup old recipe configurations from a server.  An example
-is when I changed the home directory for my web server.  I first wrote a
-cleanup that cleaned-up the old configuration, then an apply to apply the new
-configuration.  That way you can always run the apply in the future when
-building new machines and don't need the cleanup logic since the new machine
-never had the old configuration that had to get cleaned-up.
-
 file set copying
 ================
 
 The Recipe class defines a few helper functions for handling templates and
-copying files to servers.  It runs files with a .tmplt extension through make
+copying files to servers.  It runs files with a .tmplt extension through Mako,
 using the dictionary you pass to it.  Regular files just get copied.  You can
 specify owner, group, and permissions on a per-directory and per-file basis.
 
 git repo checkouts
 ==================
 
+The Recipe class also defines some helper functions for working with git repos.
 You can checkout a git repo onto the remote machine, or check it out locally
-and copy it to the remote machine.
+and copy it to the remote machine if you don't want to setup the remote machine
+to be able to do checkouts.
 '''
 import os
 import os.path
@@ -94,17 +85,17 @@ from mako.lookup import TemplateLookup
 
 class RecipeException(Exception):
     '''
-    Exception raised for exceptional conditions encountered while using the
-    Recipe class.
+    A RecipeException exception is raised for exceptional conditions
+    encountered while using the Recipe class.
     '''
     pass
 
 class FileMetaDataTracker(object):
     '''
-    Keep track of owner, group, and perms for files and directories during a
-    push_package_fileset operation.  This hinges on a file named
-    fck_metadata.txt being encountered in the directory being examined.  This
-    file should have each line contain the text
+    A FileMetaDataTracker object keeps track of owner, group, and permissions
+    for files and directories during a push_package_fileset operation.  This
+    hinges on a file named fck_metadata.txt being encountered in the directory
+    being examined.  This file should have each line contain the text
     <filename>:<owner>:<group>:<perms>, where <filename> is either a file name
     or '.' for the directory itself, <owner> is the owner's account name,
     <group> is the group's name, and <perms> is the permissions string..
@@ -114,16 +105,18 @@ class FileMetaDataTracker(object):
 
     def __init__(self):
         '''
-        Start with no metadata info.
+        Start with a blank metadata dictionary.
         '''
         self.metadata = {}
 
     def check_directory(self, root, dirs, files):
         '''
         Read in the metadata for the given directory from the fck_metadata.txt
-        file in the directory, otherwise remember metadata from previous calls.
+        file in the directory, or remember metadata from previous calls if no
+        fck_metadata.txt file is encountered.
 
-        The input parameters are what is returned from os.walk()
+        The input parameters are expected to be the values returned from a call
+        to os.walk()
 
         @type root: string
 
@@ -174,7 +167,7 @@ class FileMetaDataTracker(object):
         @type filename: string
 
         @param filename: name of file to get metadata for (leave blank if just
-        getting direcgtory metadata)
+        getting directory metadata)
 
         @rtype: tuple of strings
 
@@ -190,12 +183,44 @@ class FileMetaDataTracker(object):
 
 class Recipe(object):
     '''
-    Base object for all recipes to subclass.  It defines the framework for
-    recipes and some helper functions.  By itself it doesn't do much.
+    The Recipe class is the base class for all recipes to subclass.  It defines
+    the framework for recipes and some helper functions.  By itself it doesn't
+    do much.
+
+    Here is an example that implements a basic recipe to push a package
+    fileset::
+
+      from frycook import Recipe
+
+      class MyOwnRecipe(Recipe):
+
+          def apply(self, computer):
+              self.push_package_file_set('my_own', computer)
+
+    All recipe files should live in a single recipes package.  The __init__.py
+    file for this package should import all recipes and have a list of them so
+    they can easily be imported by the frycooker program.
+
+    Example::
+
+      from hosts import RecipeHosts
+      from nginx import RecipeNginx
+      from root import RecipeRootUser
+      from example_com import RecipeExampleCom
+
+      recipes = {
+          'hosts': RecipeHosts,
+          'nginx': RecipeNginx,
+          'root': RecipeRootUser,
+          'example_com': RecipeExacmpleCom
+      }
     '''
 
     def __init__(self, settings, environment):
         '''
+        Initialize the recipe object with the settings and environment
+        dictionaries.
+
         @type settings: dict
 
         @param settings: settings dictionary
@@ -218,12 +243,12 @@ class Recipe(object):
 
     def handle_pre_apply_message(self, computer):
         '''
-        Print the pre-apply message to the user and wait for him/her to hit
+        Print the pre-apply message for the user and wait for him/her to hit
         return before continuing.
 
         @type computer: string
 
-        @param computer: computer to apply recipe checks to
+        @param computer: computer to print pre-apply message for
         '''
         if self.pre_apply_message:
             header = "pre-apply message from %s:" % self.__class__.__name__
@@ -237,12 +262,12 @@ class Recipe(object):
 
     def handle_post_apply_message(self, computer):
         '''
-        Print the post-apply message to the user and wait for him/her to hit
+        Print the post-apply message for the user and wait for him/her to hit
         return before continuing.
 
         @type computer: string
 
-        @param computer: computer to apply recipe checks to
+        @param computer: computer to print post-apply message for
         '''
         if self.post_apply_message:
             header = "post-apply message from %s:" % self.__class__.__name__
@@ -259,8 +284,9 @@ class Recipe(object):
         This is a good place to check other things in your environment
         dictionary that the apply function expects to be there.  Override this
         function in your subclass of Recipe if you have any checks to perform.
-        Be sure to call the base class function to make sure its checks get
-        done as well.
+        Raise a RecipeException if you encounter exceptional conditions.  Be
+        sure to call the base class function to make sure its checks get done
+        as well.
 
         @type computer: string
 
@@ -292,6 +318,10 @@ class Recipe(object):
         Run the apply sequence of functions.  This is typically called by
         frycooker.
 
+        Sequence::
+
+          pre_apply_message -> pre_apply_checks() -> apply() -> post_apply_message
+
         @type computer: string
 
         @param computer: computer to apply recipe to
@@ -303,49 +333,23 @@ class Recipe(object):
 
     def run_messages(self, computer):
         '''
-        Print the pre and post apply messages only, as if the apply had
-        been run.
+        Print the pre and post apply messages only, as if the apply had been
+        run.
 
         @type computer: string
 
-        @param computer: computer to apply recipe to
+        @param computer: computer to show messages for
         '''
         self.handle_pre_apply_message(computer)
         self.handle_post_apply_message(computer)
 
-    #########################
-    ######## CLEANUP ########
-    #########################
-
-    def cleanup(self, computer):
-        '''
-        Define cleanup actions that need to be taken on the designated
-        computer.  Override this function in your sub-class of Recipe to do
-        things.
-
-        @type computer: string
-
-        @param computer: computer to apply recipe cleanup to
-        '''
-        pass
-
-    def run_cleanup(self, computer):
-        '''
-        Run the cleanup function.  This is typically called by frycooker.
-
-        @type computer: string
-
-        @param computer: computer to apply recipe cleanup to
-        '''
-        self.cleanup(computer)
-
     ###############################
     ######## FILE HANDLING ########
     ###############################
 
     def get_local_file_perms(self, local_name):
         '''
-        Get a string of the permissions set on the local file.
+        Get a string of the permissions set on a local file.
 
         @type local_name: string
 
@@ -425,7 +429,7 @@ class Recipe(object):
 
         @type perms: string
 
-        @param perms: permissions for the file
+        @param perms: permissions for the templated file
         '''
         mytemplate = self.mylookup.get_template(templatename)
         buff = mytemplate.render(**enviro)
@@ -438,18 +442,9 @@ class Recipe(object):
 
     def _push_package_file_set(self, package_name, template_env):
         '''
-        Copy a set of files to a remote server, maintaining the same directory
-        structure and processing any templates it encounters.
-
-        This copies the files to the root of the destination, so that things
-        like /etc/hosts or /etc/nginx/nginx.conf get put in the right place.
-        Just create a directory structure that mirrors the target machine and
-        all files will get copied there in the correct place.
-
-        If a file named fck_metadata.txt is encountered in a directory, then
-        it's expected to have contents of <owner>:<group> that specifies the
-        owner and group of the directory it's in, all the files in that
-        directory, and everything in its child directories.
+        Implement the file copying portion of the push_package_file_set
+        operation.  The calling function sets up the template environment, then
+        calls this one.
 
         @type package_name: string
 
@@ -485,22 +480,26 @@ class Recipe(object):
     def push_package_file_set(self, package_name, computer_name, aux_env=None):
         '''
         Copy a set of files to a remote server, maintaining the same directory
-        structure and processing any templates it encounters.
+        structure and processing any templates encountered. This copies the
+        files to the root of the destination, so that things like /etc/hosts or
+        /etc/nginx/nginx.conf get put in the right place.
 
-        This copies the files to the root of the destination, so that things
-        like /etc/hosts or /etc/nginx/nginx.conf get put in the right place.
-        Just create a directory structure that mirrors the target machine and
-        all files will get copied there in the correct place.
+        First create a directory with the same name as package_name in your
+        root packages directory.  Then create a directory structure under that
+        that mirrors the target machine and all files will get copied there in
+        the correct place.
 
         For template file processing, a default environment dictionary will be
         passed in consisting of::
 
           {"computer": host_env["computers"][computer_name]}
 
+        Template files have a .tmplt extension.
+
         If aux_env is given, it will be added to the default dictionary using
-        dict.update(), after the default is constructed.  This means that if
-        you pass in an aux_env dictionary with a key called "computer", that
-        that key will over-write the default key of that name.
+        dict.update() after the default dictionary is constructed.  This means
+        that if you pass in an aux_env dictionary with a key called "computer",
+        that that key will over-write the default key of that name.
 
         If a file named fck_metadata.txt is encountered in a directory, then
         it's expected to have contents of lines consisting of
@@ -532,8 +531,8 @@ class Recipe(object):
 
     def push_git_repo(self, computer, user, group, git_url, target_path):
         '''
-        This makes a local clone of the repo in the temp directory specified in
-        the settings file, then rsyncs this to the remote path.
+        Make a local clone of the repo in git_url into the temp directory
+        specified in the settings file, then rsync it to the remote path.
 
         @type computer: string
 
